from flask import Flask, abort, request
from tempfile import NamedTemporaryFile

import sys

import config

from queue_stack import QueueStack
from queue_stack.strategies import RecognizeAndSendStrategy

from recognizer import Recognizer
from recognizer.strategies import WhisperStrategy, FastWhisperStrategy

from message import MessageService
from message.strategies import SosMessageStrategy, NumberMessageStrategy

app = Flask(__name__)

whisper_recognizer = Recognizer(WhisperStrategy())
fast_whisper_recognizer = Recognizer(FastWhisperStrategy())

sos_message_service = MessageService(SosMessageStrategy())
number_message_service = MessageService(NumberMessageStrategy())

queue_stack = QueueStack(RecognizeAndSendStrategy())
queue_stack.start_loop_in_thread()

@app.route("/")
def hello():
  return "To recognize an audio file, upload it using a POST request with '/recognize' or '/recognize-number' route."

def recognize_files(message_service: MessageService):
  if not request.files:
    abort(400)

  results = []

  for filename, handle in request.files.items():
    temp = NamedTemporaryFile()
    handle.save(temp)

    results.append(queue_stack.append_and_await((
      temp, 
      whisper_recognizer, 
      message_service,
      config.HARPYIA_LANGUAGE,
      message_service.get_prompt()
    )))

  print(results, file=sys.stderr)
  return {'results': results}

@app.route('/recognize', methods=['POST'])
def recognize():
  return recognize_files(sos_message_service)

@app.route('/recognize-number', methods=['POST'])
def recognize_number():
  return recognize_files(number_message_service)