import sys

from threading import Thread, Event, Lock

from queue_stack.strategies import BaseProcessStrategy

class QueueStack:
  def __init__(self, strategy: BaseProcessStrategy) -> None:
    self._stack = []
    self._strategy = strategy
  
    self._lock = Lock()
    self._running = False

    self._last_response = None

  def append(self, args, event=None) -> None:
    with self._lock:
      self._stack.append((args, event))

  def append_and_await(self, args) -> any:
    event = Event()
    self.append(args, event=event)

    event.wait()
    event.clear()

    return self._last_response

  def loop(self) -> None:
    self._running = True

    while self._running:
      with self._lock:
        if self._stack:
          print('Stack length:', len(self._stack), file=sys.stderr)
          (args, event) = self._stack.pop(0)
          self._last_response = self._process(*args)

          if event:
            event.set()

  def _process(self, *args, **kwargs) -> any:
    return self._strategy.process(*args, **kwargs)

  def start_loop_in_thread(self) -> None:
    thread = Thread(target=self.loop)
    thread.start()

  def stop_loop(self) -> None:
    self._running = False